@@ -1,7 +1,8 @@
 #!/usr/bin/env python
 """Implement the Rock class and other core rocks functionality."""
 import datetime as dt
-from typing import List
+import keyword
+from typing import List, Generator
 import warnings
 
 import numpy as np
@@ -37,6 +38,11 @@ ALIASES = {
         "parameters.physical.taxonomy": "taxonomy",
         "parameters.physical.thermal_inertia": "thermal_inertia",
     },
+    "eq_state_vector": {
+        "parameters.eq_state_vector.ref_epoch": "ref_epoch",
+        "parameters.eq_state_vector.position": "position",
+        "parameters.eq_state_vector.velocity": "velocity",
+    },
     "orbital_elements": {
         "a": "semi_major_axis",
         "e": "eccentricity",
@@ -57,21 +63,86 @@ ALIASES = {
     },
 }
 
-# The lowest level in the ssoCard tree is the Value
+
+def add_paths(cls, values, parent):
+    values["path"] = parent
+
+    for name, value in values.items():
+        if isinstance(value, (Parameter, FloatValue, StringValue, IntegerValue)):
+
+            if keyword.iskeyword(name.strip("_")):
+                name = name.strip("_")
+
+            if any(
+                parameter in parent for parameter in ["color", "phase_function", "spin"]
+            ):
+                value.path = f"{parent}.<id>.{name}"
+            else:
+                value.path = f"{parent}.{name}"
+    return values
+
+
+# The lowest level in the ssoCard tree is the are the differnt Values and the Error
 class Error(pydantic.BaseModel):
     min_: float = pydantic.Field(np.nan, alias="min")
     max_: float = pydantic.Field(np.nan, alias="max")
 
 
-class Value(pydantic.BaseModel):
-    label: str = ""
-    format: str = ""
-    symbol: str = ""
-    description: str = ""
+# The second lowest level is the Parameter. Values inherit from Parameter.
+class Parameter(pydantic.BaseModel):
+
+    _label: str = pydantic.Field("", exclude=True)
+    _format: str = pydantic.Field("", exclude=True)
+    _symbol: str = pydantic.Field("", exclude=True)
+    _description: str = pydantic.Field("", exclude=True)
+    path: str = pydantic.Field("", exclude=True)
+
+    def __str__(self):
+        breakpoint()
+        return self.json()
+
+    @property
+    def label(self):
+        return self._label
+
+    @label.getter
+    def label(self):
+        label = rocks.utils.load_mappings()[self.path]["label"]
+        return label
+
+    @property
+    def symbol(self):
+        return self._symbol
+
+    @symbol.getter
+    def symbol(self):
+        if "symbol" in rocks.utils.load_mappings()[self.path]:
+            return rocks.utils.load_mappings()[self.path]["symbol"]
+        return ""
+
+    @property
+    def format(self):
+        return self._format
+
+    @format.getter
+    def format(self):
+        if "format" in rocks.utils.load_mappings()[self.path]:
+            return rocks.utils.load_mappings()[self.path]["format"]
+        return ""
+
+    @property
+    def description(self):
+        return self._description
+
+    @description.getter
+    def description(self):
+        if "description" in rocks.utils.load_mappings()[self.path]:
+            return rocks.utils.load_mappings()[self.path]["description"]
+        return ""
 
 
-class FloatValue(Value):
-    unit: str = ""
+class FloatValue(Parameter):
+    _unit: str = ""
     error: Error = Error(**{})  # min_ and max_ values
     value: float = np.nan
     error_: float = np.nan  # average of min_ and max_
@@ -91,6 +162,16 @@ class FloatValue(Value):
             return False
         return True
 
+    @property
+    def unit(self):
+        return self._unit
+
+    @unit.getter
+    def unit(self):
+        if "unit" in rocks.utils.load_mappings()[self.path]:
+            return rocks.utils.load_mappings()[self.path]["unit"]
+        return ""
+
     @pydantic.root_validator(pre=True)
     def _compute_mean_error(cls, values):
 
@@ -103,8 +184,8 @@ class FloatValue(Value):
         return values
 
 
-class IntegerValue(Value):
-    unit: str = ""
+class IntegerValue(Parameter):
+    _unit: str = ""
     value: int = None
 
     def __str__(self):
@@ -114,9 +195,19 @@ class IntegerValue(Value):
     def __bool__(self):
         return bool(self.value)
 
+    @property
+    def unit(self):
+        return self._unit
 
-class StringValue(Value):
+    @unit.getter
+    def unit(self):
+        unit = rocks.utils.load_mappings()[self.path]["unit"]
+        return unit
+
+
+class StringValue(Parameter):
     value: str = ""
+    path: str = pydantic.Field("", exclude=True)
 
     def __str__(self):
         return self.value
@@ -125,27 +216,14 @@ class StringValue(Value):
         return bool(self.value)
 
 
-class ListValue(Value):
+class ListValue(Parameter):
     value: list = []
 
-    def __str__(self):
-        return self.value
-
     def __bool__(self):
         return bool(self.value)
 
 
-# The second lowest level is the Parameter
-class Parameter(pydantic.BaseModel):
-
-    label: str = ""
-    description: str = ""
-
-    def __str__(self):
-        return self.json()
-
-
-# Other common branches are bibred, links and method
+# Other common branches are bibref, links and method
 class Method(Parameter):
     doi: str = ""
     name: str = ""
@@ -169,24 +247,43 @@ class LinksParameter(Parameter):
 
 
 # And a special class for the Spin list
-class SpinList(list):
+class ListWithAttributes(list):
     """Subclass of <list> with a custom __str__ for the Spin parameters."""
 
-    def __init__(self, list_):
+    def __init__(self, values):
         """Convert the list items to Spin instances."""
 
-        if list_:  # spin list is populated
-            list_ = [Spin(**entry) for entry in list_]
+        if isinstance(values, Generator):
+            values = list(values)
+
+        if values:  # is the list of parameters populated?
+            if isinstance(values[0], dict):  # are we instantiating the parameter class
+                if values:
+                    list_ = [type(values[0])(**entry) for entry in values]
+                else:
+                    list_ = [type(values[0])(**{})]  # ensure its never empty
+            else:
+                list_ = values
         else:
-            list_ = [Spin(**{})]  # ensure it's never empty
+            list_ = []
 
         return super().__init__(list_)
 
-    def __str__(self) -> str:
-        return "\n".join(entry.json() for entry in self)
-
-    def __bool__(self) -> bool:
-        return any(np.isfinite(entry.period.value) for entry in self)
+    def __getattr__(self, name):
+        """Implement attribute shortcuts. Gets called if __getattribute__ fails."""
+        # If it's a user attribute, we handle it here
+        if not name.startswith("__"):
+            if self:
+                if hasattr(self[0], name):
+                    return ListWithAttributes(
+                        # type(getattr(self[0], name)),
+                        [getattr(solution, name) for solution in self]
+                    )
+                else:
+                    raise AttributeError(f"Parameter does not have attribute {name}")
+            else:
+                return []
+        raise AttributeError
 
 
 # ------
@@ -197,7 +294,7 @@ class OrbitalElements(Parameter):
     ceu: FloatValue = FloatValue(**{})
     links: LinksParameter = LinksParameter(**{})
     author: StringValue = StringValue(**{})
-    bibref: List[Bibref] = [Bibref(**{})]
+    bibref: ListWithAttributes = [Bibref(**{})]
     ceu_rate: FloatValue = FloatValue(**{})
     ref_epoch: FloatValue = FloatValue(**{})
     inclination: FloatValue = FloatValue(**{})
@@ -211,6 +308,14 @@ class OrbitalElements(Parameter):
     number_observation: FloatValue = FloatValue(**{})
     perihelion_argument: FloatValue = FloatValue(**{})
 
+    @pydantic.root_validator()
+    def _add_paths(cls, values):
+        return add_paths(cls, values, "parameters.dynamical.orbital_elements")
+
+    _convert_list_to_parameterlist: classmethod = pydantic.validator(
+        "bibref", allow_reuse=True, pre=True
+    )(lambda list_: ListWithAttributes([Bibref(**element) for element in list_]))
+
 
 class ProperElements(Parameter):
     links: LinksParameter = LinksParameter(**{})
@@ -225,6 +330,10 @@ class ProperElements(Parameter):
     proper_frequency_nodal_longitude: FloatValue = FloatValue(**{})
     proper_frequency_perihelion_longitude: FloatValue = FloatValue(**{})
 
+    @pydantic.root_validator()
+    def _add_paths(cls, values):
+        return add_paths(cls, values, "parameters.dynamical.proper_elements")
+
 
 class Family(Parameter):
     links: LinksParameter = LinksParameter(**{})
@@ -240,6 +349,10 @@ class Family(Parameter):
         else:
             return "No family membership known."
 
+    @pydantic.root_validator()
+    def _add_paths(cls, values):
+        return add_paths(cls, values, "parameters.dynamical.family")
+
 
 class Pair(Parameter):
     age: FloatValue = FloatValue(**{})
@@ -247,12 +360,20 @@ class Pair(Parameter):
     sibling_name: StringValue = StringValue(**{})
     sibling_number: IntegerValue = IntegerValue(**{})
 
+    @pydantic.root_validator()
+    def _add_paths(cls, values):
+        return add_paths(cls, values, "parameters.dynamical.pair")
+
 
 class TisserandParameter(Parameter):
     jupiter: FloatValue = pydantic.Field(FloatValue(**{}), alias="Jupiter")
     method: List[Method] = [Bibref(**{})]
     bibref: List[Bibref] = [Method(**{})]
 
+    @pydantic.root_validator()
+    def _add_paths(cls, values):
+        return add_paths(cls, values, "parameters.dynamical.tisserand_parameter")
+
 
 class Yarkovsky(Parameter):
     S: FloatValue = FloatValue(**{})
@@ -265,6 +386,10 @@ class Yarkovsky(Parameter):
     def __str__(self):
         return "\n".join([self.A2.__str__(), self.dadt.__str__()])
 
+    @pydantic.root_validator()
+    def _add_paths(cls, values):
+        return add_paths(cls, values, "parameters.dynamical.yarkovsky")
+
 
 class DynamicalParameters(Parameter):
     pair: Pair = pydantic.Field(Pair(**{}), alias="pairs")
@@ -285,6 +410,8 @@ class Albedo(FloatValue):
     bibref: List[Bibref] = []
     method: List[Method] = []
 
+    path = "parameters.physical.albedo.albedo"
+
 
 class ColorEntry(Parameter):
     color: FloatValue = FloatValue(**{})
@@ -300,6 +427,10 @@ class ColorEntry(Parameter):
     id_filter_1: StringValue = StringValue(**{})
     id_filter_2: StringValue = StringValue(**{})
 
+    @pydantic.root_validator()
+    def _add_paths(cls, values):
+        return add_paths(cls, values, "parameters.physical.colors.<id>.color")
+
 
 class Color(Parameter):
     g_i: ColorEntry = pydantic.Field(ColorEntry(**{}), alias="g-i")
@@ -328,16 +459,32 @@ class Color(Parameter):
     V_R: ColorEntry = pydantic.Field(ColorEntry(**{}), alias="V-R")
     B_V: ColorEntry = pydantic.Field(ColorEntry(**{}), alias="B-V")
 
+    @pydantic.root_validator()
+    def _add_paths(cls, values):
+        return add_paths(cls, values, "parameters.physical.colors")
+
 
 class Density(FloatValue):
     method: List[Method] = []
     bibref: List[Bibref] = []
 
+    @pydantic.root_validator()
+    def _add_paths(cls, values):
+        return add_paths(cls, values, "parameters.physical.density.density")
+
 
 class Diameter(FloatValue):
     links: LinksParameter = LinksParameter(**{})
     method: List[Method] = [Method(**{})]
-    bibref: List[Bibref] = [Bibref(**{})]
+    bibref: ListWithAttributes = [Bibref(**{})]
+
+    @pydantic.root_validator()
+    def _add_paths(cls, values):
+        return add_paths(cls, values, "parameters.physical.diameter.diameter")
+
+    _convert_list_to_parameterlist: classmethod = pydantic.validator(
+        "bibref", allow_reuse=True, pre=True
+    )(lambda list_: ListWithAttributes([Bibref(**element) for element in list_]))
 
 
 class Mass(FloatValue):
@@ -345,6 +492,10 @@ class Mass(FloatValue):
     bibref: List[Bibref] = [Bibref(**{})]
     method: List[Method] = [Method(**{})]
 
+    @pydantic.root_validator()
+    def _add_paths(cls, values):
+        return add_paths(cls, values, "parameters.physical.mass.mass")
+
 
 class Phase(Parameter):
     H: FloatValue = FloatValue(**{})
@@ -379,6 +530,7 @@ class PhaseFunction(Parameter):
 
         if name in ALIASES["phase_function"].keys():
             return getattr(self, ALIASES["phase_function"][name])
+        raise AttributeError
 
     def __bool__(self):
         return any(
@@ -392,19 +544,23 @@ class PhaseFunction(Parameter):
             ]
         )
 
-    def __repr__(self):
+    def __str__(self):
         observed = []
 
         for filter_ in ["generic_johnson_V", "misc_atlas_cyan", "misc_atlas_orange"]:
             entry = getattr(self, filter_)
             if not np.isnan(entry.H.value):
                 observed.append(
-                    rf"H: {entry.H.value:.2f}  G1: {entry.G1.value:.2f}  G2: {entry.G2.value:.2f}  \[{filter_}]"
+                    rf"H: {entry.H.value:.2f}  G1: {entry.G1.value:.2f}  G2: {entry.G2.value:.2f}  [{filter_}]"
                 )
         if observed:
             return "\n".join(observed)
         return "No phase function on record."
 
+    @pydantic.root_validator()
+    def _add_paths(cls, values):
+        return add_paths(cls, values, "parameters.physical.phase_functions")
+
 
 class Spin(Parameter):
     t0: FloatValue = FloatValue(**{})
@@ -421,6 +577,10 @@ class Spin(Parameter):
     bibref: List[Bibref] = [Bibref(**{})]
     technique: StringValue = StringValue(**{})
 
+    @pydantic.root_validator()
+    def _add_paths(cls, values):
+        return add_paths(cls, values, "parameters.physical.spins")
+
 
 class Taxonomy(Parameter):
     links: LinksParameter = LinksParameter(**{})
@@ -440,6 +600,10 @@ class Taxonomy(Parameter):
             return "No taxonomy on record."
         return self.class_.value
 
+    @pydantic.root_validator()
+    def _add_paths(cls, values):
+        return add_paths(cls, values, "parameters.physical.taxonomy")
+
 
 class ThermalInertia(FloatValue):
     dsun: FloatValue = FloatValue(**{})
@@ -447,16 +611,27 @@ class ThermalInertia(FloatValue):
     bibref: List[Bibref] = [Bibref(**{})]
     method: List[Method] = [Method(**{})]
 
+    @pydantic.root_validator()
+    def _add_paths(cls, values):
+        return add_paths(
+            cls, values, "parameters.physical.thermal_inertia.thermal_inertia"
+        )
+
 
 class AbsoluteMagnitude(FloatValue):
     G: FloatValue = FloatValue(**{})
     bibref: List[Bibref] = [Bibref(**{})]
-    links: LinksParameter = LinksParameter(**{})
+
+    @pydantic.root_validator()
+    def _add_paths(cls, values):
+        return add_paths(
+            cls, values, "parameters.physical.absolute_magnitude.absolute_magnitude"
+        )
 
 
 class PhysicalParameters(Parameter):
     mass: Mass = Mass(**{})
-    spin: SpinList = SpinList([])
+    spin: ListWithAttributes = [Spin(**{})]
     color: Color = pydantic.Field(Color(**{}), alias="colors")
     albedo: Albedo = Albedo(**{})
     density: Density = Density(**{})
@@ -470,7 +645,7 @@ class PhysicalParameters(Parameter):
 
     _convert_list_to_parameterlist: classmethod = pydantic.validator(
         "spin", allow_reuse=True, pre=True
-    )(lambda list_: SpinList(list_))
+    )(lambda list_: ListWithAttributes([Spin(**element) for element in list_]))
 
 
 # ------
@@ -480,6 +655,10 @@ class EqStateVector(Parameter):
     position: ListValue = ListValue(**{})
     velocity: ListValue = ListValue(**{})
 
+    @pydantic.root_validator()
+    def _add_paths(cls, values):
+        return add_paths(cls, values, "parameters.eq_state_vector")
+
 
 # ------
 # Highest level branches
@@ -533,7 +712,7 @@ class Rock(pydantic.BaseModel):
     mpcatobs: rocks.datacloud.Mpcatobs = rocks.datacloud.Mpcatobs(**{})
     mpcorb: rocks.datacloud.Mpcorb = rocks.datacloud.Mpcorb(**{})
     pairs: rocks.datacloud.Pairs = rocks.datacloud.Pairs(**{})
-    proper_elements: rocks.datacloud.Proper_elements = rocks.datacloud.Proper_elements(
+    proper_elements_: rocks.datacloud.Proper_elements = rocks.datacloud.Proper_elements(
         **{}
     )
     phase_functions: rocks.datacloud.Phase_function = rocks.datacloud.Phase_function(
@@ -624,13 +803,6 @@ class Rock(pydantic.BaseModel):
 
             else:
 
-                # rename 'spins' to 'spin' so it does not shadow the datacloud property
-                # TODO this should be done at the parameter-name level
-                if "spins" in ssocard["parameters"]["physical"]:
-                    ssocard["parameters"]["physical"]["spin"] = ssocard["parameters"][
-                        "physical"
-                    ]["spins"]
-
                 if datacloud is not None:
                     for catalogue in datacloud:
                         ssocard = self.__add_datacloud_catalogue(
@@ -726,6 +898,9 @@ class Rock(pydantic.BaseModel):
         if name in ALIASES["dynamical"].values():
             return getattr(self.parameters.dynamical, name)
 
+        if name in ALIASES["eq_state_vector"].values():
+            return getattr(self.parameters.eq_state_vector, name)
+
         # TODO This could be coded in a more abstract way
         # These are proper aliases
         if name in ALIASES["orbital_elements"].keys():
