@@ -169,15 +169,6 @@ def _postprocess_ssocard(card):
     # Turn low-level parameters into dictionaries
     card["parameters"] = make_dict(card["parameters"])
 
-    # Catch for spins
-    if "spins" in card["parameters"]["physical"]:
-        for id_ in card["parameters"]["physical"]["spins"]:
-            mappings["parameters"]["physical"]["spins"][id_] = mappings["parameters"][
-                "physical"
-            ]["spins"]["<id>"]
-
-    card = deep_update(card, mappings)
-
     # ------
     # Convert spin to list
     if "spins" not in card["parameters"]["physical"]:
@@ -198,7 +189,8 @@ def _postprocess_ssocard(card):
         }
 
         spin_solutions.append(spin)
-    card["parameters"]["physical"]["spins"] = spin_solutions
+    card["parameters"]["physical"]["spin"] = spin_solutions
+    del card["parameters"]["physical"]["spins"]
 
     return card
 
